package cn.com.qmth.examcloud.core.oe.student.api.controller.client;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.student.bean.*;
import cn.com.qmth.examcloud.core.oe.student.service.*;
import cn.com.qmth.examcloud.support.Constants;
import cn.com.qmth.examcloud.support.examing.*;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Api(tags = "（客户端）考试过程相关接口")
@RestController
@RequestMapping("${app.api.oe.student}/client/exam/process")
public class ExamProcessController extends ControllerSupport {

    private static final Logger log = LoggerFactory.getLogger(ExamProcessController.class);

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamRecordPaperStructService examRecordPaperStructService;

    @Autowired
    private ExamControlService examControlService;

    @Autowired
    private ExamingSessionService examingSessionService;

    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Autowired
    private RedisClient redisClient;

    @ApiOperation(value = "开始考试")
    @PostMapping("/startExam")
    public StartExamInfo startExam(@RequestParam Long examStudentId) {
        User user = getAccessUser();
        return examControlService.startExam(examStudentId, user.getUserId(), getIp(getRequest()));
    }

    @ApiOperation(value = "开始答题")
    @PostMapping("/startAnswer")
    public StartAnswerInfo startAnswer(@RequestParam @ApiParam(value = "考试记录ID") Long examRecordDataId) {
        User user = getAccessUser();
        return examControlService.startAnswer(examRecordDataId, user.getUserId());
    }

    @ApiOperation(value = "断点续考:检查正在进行中的考试")
    @PostMapping("/checkExamInProgress")
    public ExamProcessResultInfo checkExamInProgress() {
        User user = getAccessUser();
        return examControlService.checkExamInProgress2(user.getUserId(), getIp(getRequest()));
    }

    @ApiOperation(value = "考试心跳")
    @PostMapping("/examHeartbeat")
    public Long examHeartbeat(HttpServletRequest request) {
        User user = getAccessUser();
        return examControlService.examHeartbeat(user, getIp(request));
    }

    @ApiOperation(value = "结束考试：交卷")
    @PostMapping("/endExam")
    public void endExam(HttpServletRequest request) {
        User user = getAccessUser();
        examControlService.manualEndExam(user.getUserId(), getIp(request));
    }

    @ApiOperation(value = "获取考试记录信息")
    @PostMapping("/getEndExamInfo")
    public EndExamInfo getEndExamInfo(@RequestParam Long examRecordDataId) {
        return examControlService.getEndExamInfo(examRecordDataId);
    }

    @ApiOperation(value = "获取考试记录试卷结构")
    @PostMapping("/getExamRecordPaperStruct")
    public ExamRecordPaperStruct getExamRecordPaperStruct(@RequestParam Long examRecordDataId) {
        return examRecordPaperStructService.getExamRecordPaperStruct(examRecordDataId);
    }

    @ApiOperation(value = "考试过程中-获取试题列表")
    @PostMapping("/findExamQuestionList")
    public List<ExamQuestion> findExamQuestionList() {
        User user = getAccessUser();
        ExamingSession examSessionInfo = examingSessionService.getExamingSession(user.getUserId());
        if (examSessionInfo == null
                || examSessionInfo.getExamingStatus().equals(ExamingStatus.INFORMAL)) {
            throw new StatusException("1001", "考试会话已过期，请重新开考");
        }

        String examingHeartbeatKey = RedisKeyHelper.getBuilder().examingHeartbeatKey(examSessionInfo.getExamRecordDataId());
        ExamingHeartbeat examingHeartbeat = redisClient.get(examingHeartbeatKey, ExamingHeartbeat.class);

        if (null != examingHeartbeat
                && examingHeartbeat.getCost() >= examSessionInfo.getExamDuration()) {
            throw new StatusException("1001", "考试会话已过期，请重新开考");
        }

        ExamRecordQuestions qers = examRecordQuestionsService.getExamRecordQuestions(examSessionInfo.getExamRecordDataId());
        List<ExamQuestion> examQuestionList = qers.getExamQuestions();
        for (ExamQuestion examQuestion : examQuestionList) {
            examQuestion.setCorrectAnswer(null);
            examQuestion.setStudentScore(null);
        }
        return examQuestionList;
    }

    @ApiOperation(value = "考试过程中-获取试题内容")
    @PostMapping("/getQuestionContent")
    public String getQuestionContent(@RequestParam String questionId) {
        User user = getAccessUser();
        return examRecordQuestionsService.getQuestionContentForClient(user.getUserId(), questionId);
    }

    @ApiOperation(value = "考试过程中-考生试题作答")
    @PostMapping("/submitQuestionAnswer")
    public void submitQuestionAnswer(@RequestBody List<ExamStudentQuestionInfo> examQuestionInfos, HttpServletRequest request) {
        User user = getAccessUser();
        String referer = request.getHeader("REFERER");
        String agent = request.getHeader("USER-AGENT");
        examRecordQuestionsService.submitQuestionAnswer(user.getUserId(), examQuestionInfos, referer, agent);
    }

    @ApiOperation(value = "保存活体检测结果")
    @PostMapping("/saveFaceBiopsyResult")
    public SaveFaceBiopsyResultResp saveFaceBiopsyResult(@RequestBody SaveFaceBiopsyResultReq req) {
        // todo
        return null;
    }

    @ApiOperation(value = "获取课程名称")
    @PostMapping("/courseName/{id}")
    public String courseName(@PathVariable Long id) {
        return examRecordDataService.findCourseNameById(id);
    }

    @ApiOperation(value = "违纪（非法考生端应用）")
    @PostMapping("/discipline")
    public void discipline(@RequestParam(required = false) String reason) {
        User user = getAccessUser();
        ExamingSession examingSession = examingSessionService.getExamingSession(user.getUserId());
        if (examingSession == null) {
            return;
        }

        if (StringUtils.isNotEmpty(reason)) {
            // 限制字符串最大长度
            reason = StringUtils.substring(reason, 0, 1000);
        }

        String cacheKey = Constants.OE_DISCIPLINE_ILLEGAL_CLIENT + examingSession.getExamRecordDataId();
        redisClient.set(cacheKey, reason, 3 * 60 * 60);
    }

}
